@@ -1,10 +1,9 @@
 from __future__ import print_function, division
-import h5py
 from .core import KoshLoader
 
 
 def walk_hdf5(dataset, prefix=""):
-    """Walk through hdf5 groups to find all datsets and return their paths
+    """Walk through hdf5 groups to find all datasets and return their paths
     return generator
     :param dataset: hdf5 dataset to start walking from
     :type dataset: h5py._hl.dataset.Dataset
@@ -12,6 +11,7 @@ def walk_hdf5(dataset, prefix=""):
     :return: hdf5 dataset structure
     :rtype: generator
     """
+    import h5py
     for key in sorted(dataset.keys()):
         value = dataset[key]
         if isinstance(value, h5py._hl.dataset.Dataset):
@@ -53,6 +53,7 @@ class HDF5Loader(KoshLoader):
         :type mode: str, optional
         :return: Kosh File object
         """
+        import h5py
         return h5py.File(self.obj.uri, mode)
 
     def extract(self):
@@ -64,8 +65,9 @@ class HDF5Loader(KoshLoader):
         :type format: str
         :return: data
         """
+        import h5py
         args, kargs = self._user_passed_parameters
-        f = h5py.File(self.obj.uri, "r")
+        f = h5py.File(self.uri, "r")
         features = self.feature
         if not isinstance(features, list):
             features = [self.feature, ]
@@ -111,7 +113,8 @@ class HDF5Loader(KoshLoader):
         :return: list of features available in file
         :rtype: list
         """
-        with h5py.File(self.obj.uri, "r") as f:
+        import h5py
+        with h5py.File(self.uri, "r") as f:
             features = list_hdf5(f)
         if group is not None:
             feats = []
@@ -136,12 +139,13 @@ class HDF5Loader(KoshLoader):
         :return: dictionary describing the feature
         :rtype: dict
         """
+        import h5py
         features = self._list_features()
         if feature not in features:
             raise ValueError("feature {feature} is not available".format(feature=feature))
 
         info = {}
-        with h5py.File(self.obj.uri, "r") as f:
+        with h5py.File(self.uri, "r") as f:
             feature = f[feature]
             info["size"] = feature.shape
             info["format"] = "hdf5"
